#!/usr/bin/env python

from elementtree.XMLTreeBuilder import TreeBuilder
from elementtree.ElementTree import ElementTree

class simple:
  """simple wrapper for xml objects"""
  def __init__(self,obj):
    """construct object from the elementtree item"""
    self.obj = obj
    self.tag = obj.tag
  def addchild(self,tag):
    """add a child with the given tag name"""
    child = simple(self.obj.find(tag))
    setattr(self,tag.lower(),child)
    return child
  def addchildren(self,tag):
    """add all children with the given tag name"""
    children = self.obj.findall(tag)
    objects = [simple(child) for child in children]
    setattr(self,tag.lower(),objects)
    return objects
  def addtext(self,tag):
    """add some contained text"""
    text = self.obj.findtext(tag)
    setattr(self,tag.lower(),text)
  def addvalues(self,tag):
    """add some contained values..."""
    children = self.obj.findall(tag)
    values = [child.text for child in children]
    setattr(self,tag.lower(),values)
  def __repr__(self):
    """return a representation of the object"""
    return self.tag+':'+repr(self.__dict__)
  def getattr(self, attrname):
    """gets an attribute of the tag"""
    return self.obj.attrib[attrname]
  def write(self, file, encoding="UTF-8"):
    """writes the object as XML to a file..."""
    e = ElementTree(self.obj)
    e.write(file, encoding)

class KimdabaConfig(simple):
  def __init__(self, configstring):
    """build the KimdabaConfig from the given string"""
    parser = TreeBuilder()
    parser.feed(configstring)
    root = parser.close()
    simple.__init__(self, root)
    self.configoptions = self.getconfigoptions()
    self.optionsdict = self.getoptionsdict()
    self.imagedict = self.getimagedict()
    self.keyworddict = self.getkeyworddict()

  def getconfigoptions(self):
    return self.addchild("config").obj.attrib

  def getoptionsdict(self):
    optionsroot = self.addchild("options")
    options = optionsroot.addchildren("option")
    optionsdict = {}
    for option in options:
      values = option.addchildren("value")
      optionsdict[option.getattr("name")] = [value.getattr("value") for value in values]
    return optionsdict

  def getmembergroups(self):
    membergroups = self.addchild("member-groups")
    membersroot = membergroups.addchildren("member")
    memberdict = {}
    for member in membersroot:
      memberdict[(member.getattr("option-group"), member.getattr("member"))] = member.getattr("group-name")
    return memberdict

  def getimagedict(self):
    images = self.addchild("images")
    imagedict = {}
    imagesroot = images.addchildren("image")
    for image in imagesroot:
      imageoptions = image.addchild("options")
      imageoptiondict = {}
      for option in imageoptions.addchildren("option"):
        optionname = option.getattr("name")
        valueroots = option.addchildren("value")
        values = [value.getattr("value") for value in valueroots]
        imageoptiondict[optionname] = values
      imagedict[image.getattr("file")] = imageoptiondict
    return imagedict

  def getkeyworddict(self):
    keyworddict = {}
    for name, values in self.optionsdict.iteritems():
      keyworddict[name] = dict([(value, []) for value in values])
    for imagename, imageoptiondict in self.imagedict.iteritems():
      for optionname, optionvalues in imageoptiondict.iteritems():
        for optionvalue in optionvalues:
          keyworddict[optionname][optionvalue].append(imagename)
    return keyworddict

  def getimagenames(self):
    return self.imagedict.keys()

  def filterimages(self, imagelist, optionname, optionvalue):
    matchingimages = self.keyworddict[optionname][optionvalue]
    return [image for image in imagelist if image in matchingimages]

  def getimages(self, filterlist):
    matchingimages = self.getimagenames()
    for optionname, optionvalue in filterlist:
      matchingimages = self.filterimages(matchingimages, optionname, optionvalue)
    return matchingimages

if __name__ == "__main__":
  import sys
  kimdabaconfig = KimdabaConfig(sys.stdin.read())
  kimdabaconfig.write(sys.stdout)
  filterlist = kimdabaconfig.getimagenames()
  filterlist = kimdabaconfig.filterimages(filterlist, "Persons", "Danielle")
  filterlist = kimdabaconfig.filterimages(filterlist, "Locations", "Vincent Pallotti")

