#!/usr/bin/env python

from wxPython.wx import *
import os
import imdaba
import threading

ID_ABOUT=101
ID_OPEN=102
ID_HOME=103
ID_BACK=104
ID_SLIDESHOW=105
ID_RUNCMD=106
ID_EXIT=110

batchsize=1000
thumbnailwidth=96
thumbnailheight=96

thumbnailsinbackground=1

# Define notification event for thread completion
EVT_THUMBNAILSDONE_ID = wxNewId()

def EVT_THUMBNAILSDONE(win, func):
    win.Connect(-1, -1, EVT_THUMBNAILSDONE_ID, func)

class ThumbnailsDoneEvent(wxPyEvent):
    """Simple event to carry arbitrary result data"""
    def __init__(self):
        wxPyEvent.__init__(self)
        self.SetEventType(EVT_THUMBNAILSDONE_ID)

# Thread class that executes processing
class WorkerThread(threading.Thread):
    def __init__(self, notify_window, call_method, event_type):
        threading.Thread.__init__(self)
        self._notify_window = notify_window
        self._call_method = call_method
        self._event_type = event_type
        self._want_abort = 0
        # This starts the thread running on creation, but you could
        # also make the GUI thread responsible for calling this
        self.start()

    def run(self):
        # This is the code executing in the new thread. Simulation of
        # a long process (well, 10s here) as a simple loop - you will
        # need to structure your processing so that you periodically
        # peek at the abort variable
        notdone = True
        while notdone:
            notdone = self._call_method()
            if self._want_abort:
                # Use a result of None to acknowledge the abort
                wxPostEvent(self._notify_window, self._event_type())
                return
        wxPostEvent(self._notify_window, self._event_type())

    def abort(self):
        # Method for use by main thread to signal an abort
        self._want_abort = 1

class SlideshowWindow(wxFrame):
    """Window for Slideshows..."""
    def __init__(self, parent, id, title, selectedimages):
        self.parent = parent
        wxFrame.__init__(self,parent,-4, title, style=wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE)
        self.selectedimages = selectedimages
        # Use some sizers to see layout options
        self.sizer=wxBoxSizer(wxVERTICAL)
        self.currentimage = wxStaticBitmap(self, -1, wxEmptyBitmap(1,1))
        self.sizer.Add(self.currentimage,1,wxEXPAND)
        EVT_KEY_DOWN(self, self.OnKeyDown)
        #Layout sizers
        self.SetSizer(self.sizer)
        self.SetAutoLayout(1)
        self.sizer.Fit(self)
        self.ShowFullScreen(True)
        self.Show()
        self.SetIndex(0)

    def SetIndex(self, index):
        if index < 0: index = 0
        if index >= len(self.selectedimages): index = len(self.selectedimages) - 1
        self.currentindex = index
        image = self.parent.GetImage(self.selectedimages[index])
        width, height = image.GetWidth(), image.GetHeight()
        currentwidth, currentheight = self.GetSizeTuple()
        if float(width)/height > float(currentwidth)/currentheight:
          currentheight = int(float(height)/width*currentwidth)
        else:
          currentwidth = int(float(width)/height*currentheight)
        image = image.Scale(currentwidth, currentheight)
        self.currentimage.SetBitmap(wxBitmapFromImage(image))

    def OnKeyDown(self, event):
        keycode = event.GetKeyCode()
        if keycode in (WXK_DOWN, WXK_RIGHT, WXK_NEXT, WXK_RETURN, WXK_SPACE):
          self.SetIndex(self.currentindex + 1)
        elif keycode in (WXK_UP, WXK_LEFT, WXK_PRIOR, WXK_BACK):
          self.SetIndex(self.currentindex - 1)
        elif keycode in (WXK_ESCAPE,):
          self.Close(true)
        else:
          event.Skip()

class MainWindow(wxFrame):
    """ Main wxImdaba window """
    def __init__(self,parent,id,title):
        wxFrame.__init__(self,parent,-4, title, size=(800,600),
                                     style=wxDEFAULT_FRAME_STYLE|wxNO_FULL_REPAINT_ON_RESIZE)
        self.statusbar = self.CreateStatusBar()
        # Setting up the filemenu
        filemenu = wxMenu()
        filemenu.Append(ID_OPEN, "&Open", "Open an image database")
        filemenu.Append(ID_ABOUT, "&About"," Information about this program")
        filemenu.AppendSeparator()
        filemenu.Append(ID_EXIT,"E&xit"," Terminate the program")
        # and the imagemenu
        imagemenu = wxMenu()
        imagemenu.Append(ID_SLIDESHOW, "&Slideshow", "Show current images in a slideshow")
        imagemenu.Append(ID_RUNCMD, "&Run", "Run command on current images...")
        # and the filtermenu
        filtermenu = wxMenu()
        filtermenu.Append(ID_HOME, "&Home", "Home (clear filters)")
        filtermenu.Append(ID_BACK, "&Back", "Back (clear last filter)")
        # Creating the menubar.
        menuBar = wxMenuBar()
        menuBar.Append(filemenu,"&File")
        menuBar.Append(imagemenu, "&Image")
        menuBar.Append(filtermenu, "Fil&ter")
        self.SetMenuBar(menuBar)  # Adding the MenuBar to the Frame content.

        EVT_MENU(self, ID_OPEN, self.OnOpen)
        EVT_MENU(self, ID_SLIDESHOW, self.OnSlideshow)
        EVT_MENU(self, ID_RUNCMD, self.OnRunCommand)
        EVT_MENU(self, ID_ABOUT, self.OnAbout)
        EVT_MENU(self, ID_HOME, self.OnHome)
        EVT_MENU(self, ID_BACK, self.OnBack)
        EVT_MENU(self, ID_EXIT, self.OnExit)

        self.sizerbelow = wxBoxSizer(wxHORIZONTAL)
        self.imagelist = wxListCtrl(self, 1, style=wxLC_ICON|wxSUNKEN_BORDER)
        self.currentimage = wxStaticBitmap(self, -1, wxEmptyBitmap(1,1))
        self.sizerbelow.Add(self.imagelist, 1, wxEXPAND)
        self.sizerbelow.Add(self.currentimage, 1, wxEXPAND)
        self.thumbnaillist = wxImageList(thumbnailwidth, thumbnailheight, False, 0)

        self.sizerabove = wxBoxSizer(wxHORIZONTAL)
        self.categorylist = wxListCtrl(self, wxNewId(), style=wxLC_REPORT|wxSUNKEN_BORDER)
        self.optionslist = wxListCtrl(self, wxNewId(), style=wxLC_REPORT|wxSUNKEN_BORDER)
        self.sizerabove.Add(self.categorylist, 1, wxEXPAND)
        self.sizerabove.Add(self.optionslist, 1, wxEXPAND)
        
        # Use some sizers to see layout options
        self.sizer=wxBoxSizer(wxVERTICAL)
        self.sizer.Add(self.sizerabove,1,wxEXPAND)
        self.sizer.Add(self.sizerbelow,1,wxEXPAND)

        #Layout sizers
        self.SetSizer(self.sizer)
        self.SetAutoLayout(1)

        self.ResetCategoryList()
        self.ResetOptionsList()
        self.ResetImageList()
        self.selectedimages = []
        EVT_LIST_ITEM_SELECTED(self, self.categorylist.GetId(), self.OnCategorySelect)
        EVT_LIST_ITEM_SELECTED(self, self.optionslist.GetId(), self.OnOptionSelect)
        EVT_LIST_ITEM_SELECTED(self, self.imagelist.GetId(), self.OnImageSelect)
        self.thumbnailworker = None
        self.thumbnailsfound = []
        EVT_THUMBNAILSDONE(self, self.OnThumbnailsDone)
        EVT_IDLE(self, self.OnIdle)
        self.Show(true)

    def ResetImageList(self):
        self.imagelist.ClearAll()
        self.thumbnaillist.RemoveAll()
        self.thumbnaillist.Add(wxEmptyBitmap(thumbnailwidth, thumbnailheight))
        self.thumbnailsneeded = []
        self.thumbnailsfound = []
        self.imagelist.SetImageList(self.thumbnaillist, wxIMAGE_LIST_NORMAL)

    def ResetCategoryList(self):
        self.currentcategory = None
        self.categorylist.ClearAll()
        self.categorylist.InsertColumn(0, "category")
        self.categorylist.InsertColumn(1, "count")

    def ResetOptionsList(self):
        self.optionslist.ClearAll()
        self.optionslist.InsertColumn(0, "option")
        self.optionslist.InsertColumn(1, "count")

    def OnAbout(self,e):
        d = wxMessageDialog(self, "An Image Database\n"
                            "in wxPython","About wxImdaba", wxOK)
                            # Create a message dialog box
        d.ShowModal() # Shows it
        d.Destroy() # finally destroy it when finished.

    def OnSlideshow(self,e):
        slideshowwindow = SlideshowWindow(self, wxNewId(), "wxImdabaSlideshow", self.selectedimages)
        slideshowwindow.ShowFullScreen(True)

    def OnRunCommand(self,e):
        d = wxTextEntryDialog(self, "Enter the command to pass the images to...",
                              "Run command on images", "", wxOK | wxCANCEL | wxCENTRE)
        d.ShowModal()
        command = d.GetValue()
        if command:
            prevdir = os.path.abspath(os.curdir)
            if os.path.exists(command):
                abscommand = os.path.abspath(command)
            else:
                # default value will let os raise the error...
                abscommand = command
                # find the command executable on the path
                pathdirs = os.defpath.split(os.pathsep)
                for pathdir in pathdirs:
                    pathcommand = os.path.join(pathdir, command)
                    if os.path.exists(pathcommand):
                        abscommand = os.path.abspath(pathcommand)
                        break
            os.chdir(self.GetImageDirectory())
            os.spawnv(os.P_NOWAIT, abscommand, [command] + self.selectedimages)
            os.chdir(prevdir)

    def OnExit(self,e):
        if self.thumbnailworker:
            self.thumbnailworker.abort()
        self.Close(true)  # Close the frame.

    def OnThumbnailsDone(self, event):
        self.thumbnailworker = None

    def OnOpen(self,e):
        """ Open a file"""
        if not hasattr(self, "dirname"):
            self.dirname = os.curdir
        dlg = wxFileDialog(self, "Choose a file", self.dirname, "", "*.xml", wxOPEN)
        if dlg.ShowModal() == wxID_OK:
            self.filename=dlg.GetFilename()
            self.dirname=dlg.GetDirectory()
            f=open(os.path.join(self.dirname,self.filename),'r')
            self.ReadFile(f)
            f.close()
        dlg.Destroy()

    def ReadFile(self, f):
        self.ResetCategoryList()
        self.kimdabaconfig = imdaba.KimdabaConfig(f.read())
        for category, options in self.kimdabaconfig.optionsdict.iteritems():
          index = self.categorylist.GetItemCount()
          self.categorylist.InsertStringItem(index, category)
          self.categorylist.SetStringItem(index, 1, "%d Items" % len(options))
        self.Home()

    def Home(self):
        self.filters = []
        self.FilterImages()
        self.RefreshImages()

    def OnHome(self, event):
        self.Home()

    def OnBack(self, event):
        self.filters.pop()
        self.FilterImages()
        self.RefreshImages()

    def OnCategorySelect(self, event):
        self.currentcategory = event.GetText()
        self.ResetOptionsList()
        options = self.kimdabaconfig.optionsdict[self.currentcategory]
        for option in options:
          index = self.optionslist.GetItemCount()
          self.optionslist.InsertStringItem(index, option)

    def OnOptionSelect(self, event):
        option = event.GetText()
        self.filters.append((self.currentcategory, option))
        self.FilterImages()
        self.RefreshImages()

    def GetImageDirectory(self):
        return self.kimdabaconfig.configoptions["imageDirectory"]

    def GetImage(self, filename):
        filepath = os.path.join(self.GetImageDirectory(), filename)
        image = wxImage(filepath, wxBITMAP_TYPE_JPEG)
        return image

    def GetThumbnail(self, filename):
        thumbname = "%dx%d-0-%s" % (thumbnailwidth, thumbnailheight, filename)
        thumbpath = os.path.join(self.kimdabaconfig.configoptions["imageDirectory"], "ThumbNails", thumbname)
        if os.path.exists(thumbpath):
          image = wxImage(thumbpath, wxBITMAP_TYPE_JPEG)
          image = image.Scale(thumbnailwidth, thumbnailheight)
        else:
          image = self.GetImage(filename)
          image = image.Scale(thumbnailwidth, thumbnailheight)
          image.SaveFile(thumbpath, wxBITMAP_TYPE_JPEG)
        return image

    def OnImageSelect(self, event):
        filename = event.GetText()
        image = self.GetImage(filename)
        width, height = image.GetWidth(), image.GetHeight()
        currentwidth, currentheight = self.currentimage.GetSizeTuple()
        if float(width)/height > float(currentwidth)/currentheight:
          currentheight = int(float(height)/width*currentwidth)
        else:
          currentwidth = int(float(width)/height*currentheight)
        image = image.Scale(currentwidth, currentheight)
        self.currentimage.SetBitmap(wxBitmapFromImage(image))

    def RefreshImages(self):
        if self.thumbnailworker:
            self.thumbnailworker.abort()
        self.ResetImageList()
        for filename in self.selectedimages[self.batch*batchsize:(self.batch+1)*batchsize]:
            index = self.imagelist.GetItemCount()
            self.imagelist.InsertStringItem(index, filename)
            self.imagelist.SetItemImage(index, 0, 0)
            if thumbnailsinbackground:
              self.thumbnailsneeded.append((index, filename))
            else:
              self.MakeThumbnail(index, filename)
        if thumbnailsinbackground:
          self.thumbnailsneeded.reverse()
          self.thumbnailworker = WorkerThread(self, self.QueueThumbnail, ThumbnailsDoneEvent)

    def MakeThumbnail(self, index, filename):
        image = self.GetThumbnail(filename)
        imageindex = self.thumbnaillist.Add(wxBitmapFromImage(image))
        self.imagelist.SetItemImage(index, imageindex, imageindex)

    def DequeueThumbnail(self):
        if len(self.thumbnailsfound) > 0:
            index, image = self.thumbnailsfound.pop()
            imageindex = self.thumbnaillist.Add(wxBitmapFromImage(image))
            self.imagelist.SetItemImage(index, imageindex, imageindex)

    def OnIdle(self, event):
        self.DequeueThumbnail()

    def QueueThumbnail(self):
        if not len(self.thumbnailsneeded):
            return 0
        index, filename = self.thumbnailsneeded.pop()
        image = self.GetThumbnail(filename)
        self.thumbnailsfound.append((index, image))
        wxWakeUpIdle()
        return 1

    def FilterImages(self):
        statustext = " / ".join(["%s=%s" % (category, option) for category, option in self.filters])
        self.statusbar.SetStatusText(statustext)
        self.selectedimages = self.kimdabaconfig.getimages(self.filters)
        self.selectedimages.sort()
        self.batch = 0

if __name__ == "__main__":
  app = wxPySimpleApp()
  frame = MainWindow(None, -1, "wxImdaba")
  frame.Show(1)
  app.MainLoop()

